@@ -13,6 +13,7 @@
 #include "base/files/file_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
+#include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "content/public/common/url_constants.h"
 #include "extensions/common/api/content_scripts.h"
@@ -23,7 +24,10 @@
 #include "extensions/common/permissions/permissions_data.h"
 #include "extensions/common/url_pattern.h"
 #include "extensions/common/url_pattern_set.h"
+#include "extensions/common/user_script.h"
 #include "extensions/strings/grit/extensions_strings.h"
+#include "net/base/mime_util.h"
+#include "third_party/blink/public/common/mime_util/mime_util.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "url/gurl.h"
 
@@ -50,6 +54,48 @@ constexpr char kInvalidMatchDynamicScriptError[] =
 constexpr char kForbiddenInlineCodeScriptError[] =
     "Script with ID '*' has forbidden inline code source";
 
+// Returns true if the given path's mime type can be used for the given content
+// script type.
+bool IsMimeTypeValid(const base::FilePath& relative_path,
+                     ContentScriptType content_script_type) {
+  auto file_extension = relative_path.Extension();
+  if (file_extension.empty()) {
+    return false;
+  }
+
+  // Strip leading ".".
+  file_extension = file_extension.substr(1);
+
+  // Allow .user.js files, which normally have no mime type, for JS content
+  // scripts.
+  if (content_script_type == ContentScriptType::kJs &&
+      base::EqualsCaseInsensitiveASCII(file_extension,
+                                       UserScript::kFileExtension)) {
+    return true;
+  }
+
+  // Allow .scss files for CSS content scripts for compatibility with existing
+  // extensions that were using such files before the mime type check was
+  // introduced.
+  if (content_script_type == ContentScriptType::kCss &&
+      base::EqualsCaseInsensitiveASCII(file_extension, "scss")) {
+    return true;
+  }
+
+  std::string mime_type;
+  if (!net::GetWellKnownMimeTypeFromExtension(file_extension, &mime_type)) {
+    return false;
+  }
+
+  switch (content_script_type) {
+    case ContentScriptType::kJs:
+      return blink::IsSupportedJavascriptMimeType(mime_type);
+
+    case ContentScriptType::kCss:
+      return mime_type == "text/css";
+  }
+}
+
 // Returns false and sets the error if script file can't be loaded, or if it's
 // not UTF-8 encoded. If a script file can be loaded but will exceed
 // `max_script_length`, return true but add an install warning to `warnings`.
@@ -355,6 +401,28 @@ void ParseGlobs(const std::vector<std::string>* include_globs,
   }
 }
 
+bool ValidateMimeTypeFromFileExtension(const base::FilePath& relative_path,
+                                       ContentScriptType content_script_type,
+                                       std::string* error) {
+  if (!IsMimeTypeValid(relative_path, content_script_type)) {
+    switch (content_script_type) {
+      case ContentScriptType::kJs:
+        *error = l10n_util::GetStringFUTF8(
+            IDS_EXTENSION_CONTENT_SCRIPT_FILE_BAD_JS_MIME_TYPE,
+            relative_path.LossyDisplayName());
+        return false;
+
+      case ContentScriptType::kCss:
+        *error = l10n_util::GetStringFUTF8(
+            IDS_EXTENSION_CONTENT_SCRIPT_FILE_BAD_CSS_MIME_TYPE,
+            relative_path.LossyDisplayName());
+        return false;
+    }
+  }
+
+  return true;
+}
+
 bool ValidateFileSources(const UserScriptList& scripts,
                          ExtensionResource::SymlinkPolicy symlink_policy,
                          std::string* error,
@@ -370,6 +438,16 @@ bool ValidateFileSources(const UserScriptList& scripts,
         continue;
       }
 
+      // Files with invalid mime types will be ignored.
+      if (!ValidateMimeTypeFromFileExtension(js_script->relative_path(),
+                                             ContentScriptType::kJs, error)) {
+        warnings->emplace_back(
+            base::StringPrintf(errors::kInvalidUserScriptMimeType,
+                               error->c_str()),
+            api::content_scripts::ManifestKeys::kContentScripts);
+        continue;
+      }
+
       const base::FilePath& path = ExtensionResource::GetFilePath(
           js_script->extension_root(), js_script->relative_path(),
           symlink_policy);
@@ -384,6 +462,16 @@ bool ValidateFileSources(const UserScriptList& scripts,
 
     for (const std::unique_ptr<UserScript::Content>& css_script :
          script->css_scripts()) {
+      // Files with invalid mime types will be ignored.
+      if (!ValidateMimeTypeFromFileExtension(css_script->relative_path(),
+                                             ContentScriptType::kCss, error)) {
+        warnings->emplace_back(
+            base::StringPrintf(errors::kInvalidUserScriptMimeType,
+                               error->c_str()),
+            api::content_scripts::ManifestKeys::kContentScripts);
+        continue;
+      }
+
       const base::FilePath& path = ExtensionResource::GetFilePath(
           css_script->extension_root(), css_script->relative_path(),
           symlink_policy);
@@ -400,6 +488,33 @@ bool ValidateFileSources(const UserScriptList& scripts,
   return true;
 }
 
+bool ValidateUserScriptMimeTypesFromFileExtensions(const UserScript& script,
+                                                   std::string* error) {
+  for (const std::unique_ptr<UserScript::Content>& js_script :
+       script.js_scripts()) {
+    // Don't validate scripts with inline code source, since they don't have
+    // file sources.
+    if (js_script->source() == UserScript::Content::Source::kInlineCode) {
+      continue;
+    }
+
+    if (!ValidateMimeTypeFromFileExtension(js_script->relative_path(),
+                                           ContentScriptType::kJs, error)) {
+      return false;
+    }
+  }
+
+  for (const std::unique_ptr<UserScript::Content>& css_script :
+       script.css_scripts()) {
+    if (!ValidateMimeTypeFromFileExtension(css_script->relative_path(),
+                                           ContentScriptType::kCss, error)) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
 bool ValidateMatchOriginAsFallback(
     mojom::MatchOriginAsFallbackBehavior match_origin_as_fallback,
     const URLPatternSet& url_patterns,
