@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "web_engine_context.h"
 
@@ -626,7 +627,7 @@ void WebEngineContext::addProfileAdapter(ProfileAdapter *profileAdapter)
     if (content::RenderProcessHost::run_renderer_in_process()){
         if (!m_profileAdapters.isEmpty())
             qFatal("Single mode supports only single profile.");
-        // there is only one profle therefore make it 'default'
+        // there is only one profile therefore make it 'default'
         m_defaultProfileAdapter.reset(profileAdapter);
     }
     m_profileAdapters.append(profileAdapter);
@@ -665,7 +666,7 @@ void WebEngineContext::destroy()
 
     // Delete the global object and thus custom profiles
     // In case of single process ~RenderProcessHostImpl (there is only one instance)
-    // is called expliclty by BrowserMainLoop::ShutdownThreadsAndCleanUp and requires browser context.
+    // is called explicitly by BrowserMainLoop::ShutdownThreadsAndCleanUp and requires browser context.
     // therefore delete browser context on PostMainMessageLoopRun.
     if (!content::RenderProcessHost::run_renderer_in_process()) {
         m_defaultProfileAdapter.reset();
@@ -677,7 +678,7 @@ void WebEngineContext::destroy()
     }
 
     // Handle any events posted by browser-context shutdown.
-    // This should deliver all nessesery calls of DeleteSoon from PostTask
+    // This should deliver all necessary calls of DeleteSoon from PostTask
     flushMessages();
 
     m_devtoolsServer.reset();
@@ -950,19 +951,33 @@ WebEngineContext::WebEngineContext()
             qWarning("--use-gl=%s is set with --disable-gpu. Expect troubles!", glType.c_str());
     }
 
-#if BUILDFLAG(IS_OZONE) && QT_CONFIG(webengine_vulkan)
+#if BUILDFLAG(IS_OZONE)
     if (QQuickWindow::graphicsApi() == QSGRendererInterface::OpenGL && usingSupportedSGBackend()) {
         const bool disableGpu = parsedCommandLine.HasSwitch(switches::kDisableGpu);
         const bool usingVulkan = isFeatureEnabled(features::kVulkan.name, parsedCommandLine);
         if (!disableGpu && !usingVulkan && !isGbmSupported()) {
-            qWarning("GBM is not supported with the current configuration. "
-                     "Fallback to Vulkan rendering in Chromium.");
-            parsedCommandLine.AppendSwitchASCII(switches::kUseVulkan,
-                                                switches::kVulkanImplementationNameNative);
-            enableFeatures.push_back(features::kVulkan.name);
+#if QT_CONFIG(webengine_vulkan)
+            QVulkanInstance vulkanInstance;
+            vulkanInstance.setApiVersion(QVersionNumber(1, 1));
+            QRhiVulkanInitParams params;
+            params.inst = &vulkanInstance;
+
+            if (vulkanInstance.create() && QRhi::probe(QRhi::Vulkan, &params)) {
+                qWarning("GBM is not supported with the current configuration. "
+                         "Fallback to Vulkan rendering in Chromium.");
+                parsedCommandLine.AppendSwitchASCII(switches::kUseVulkan,
+                                                    switches::kVulkanImplementationNameNative);
+                enableFeatures.push_back(features::kVulkan.name);
+            } else
+#endif
+            {
+                qWarning("GBM is not supported with the current configuration and Vulkan is not "
+                         "available. Fallback to software rendering.");
+                parsedCommandLine.AppendSwitch(switches::kDisableGpu);
+            }
         }
     }
-
+#if QT_CONFIG(webengine_vulkan)
     if (QQuickWindow::graphicsApi() == QSGRendererInterface::Vulkan && usingSupportedSGBackend()) {
         // TODO: Try not to force Chromium's Vulkan backend on Linux.
         //       Currently we force it because OzoneImageBackingFactory does not support to create
@@ -992,7 +1007,8 @@ WebEngineContext::WebEngineContext()
             qputenv(deviceExtensionsVar, requiredDeviceExtensions.join(';'));
         }
     }
-#endif // BUILDFLAG(IS_OZONE) && QT_CONFIG(webengine_vulkan)
+#endif // QT_CONFIG(webengine_vulkan)
+#endif // BUILDFLAG(IS_OZONE)
 
 #if defined(Q_OS_WIN)
     if (QQuickWindow::graphicsApi() == QSGRendererInterface::Direct3D11
@@ -1208,12 +1224,20 @@ bool WebEngineContext::isGbmSupported()
             return false;
         }
 
+#if !BUILDFLAG(IS_OZONE_X11)
+        if (OzoneUtilQt::usingGLX()) {
+            qWarning("GLX: Disable GBM because Ozone X11 is not available. "
+                     "Possibly caused by missing libraries for qpa-xcb support.");
+            return false;
+        }
+#endif
+
         return true;
     }();
 
     return supported;
 }
-#endif
+#endif // BUILDFLAG(IS_OZONE)
 
 void WebEngineContext::registerMainThreadFactories()
 {
@@ -1242,7 +1266,7 @@ const char *qWebEngineChromiumVersion() noexcept
 
 const char *qWebEngineChromiumSecurityPatchVersion() noexcept
 {
-    return "140.0.7339.207"; // FIXME: Remember to update
+    return "142.0.7444.162"; // FIXME: Remember to update
 }
 
 QT_END_NAMESPACE
